import 'package:logging/logging.dart';
import 'package:rook_auth/rook_auth.dart';
import 'package:rook_users/src/data/preferences/preferences.dart';
import 'package:rook_users/src/data/server/server.dart';
import 'package:rook_users/src/domain/enums/enums.dart';
import 'package:rook_users/src/domain/model/models.dart';

/// Helper class to manage all users operations.
class RookUsersManager {
  final Logger _logger = Logger('RookUsersManager');

  bool _authorized = false;

  final UsersPreferences _preferences = UsersPreferences();
  late UsersApi api;

  /// Create a new [RookUsersManager] instance.
  ///
  /// [apiUrl] - Users API url without https.
  RookUsersManager(String apiUrl, String clientUUID, String clientPassword) {
    api = UsersApi(
      apiUrl: apiUrl,
      clientUUID: clientUUID,
      clientPassword: clientPassword,
    );
  }

  /// Checks the client is authorized to use the rookWebSockets package.
  ///
  /// Throws [AuthorizationException] if not authorized.
  Future<void> _checkAuthorization() async {
    if (!_authorized) {
      final result = await AuthorizationProvider('').getLocalAuthorization();

      _authorized = result.hasFeature(Feature.rookHealthConnect);

      if (!_authorized) {
        throw AuthorizationException();
      }
    }
  }

  /// Registers provided userID in server and saves it to preferences.
  ///
  /// If the user was previously registered it will return the stored preferences value.
  ///
  /// * [clientUUID] - Unique client identifier.
  /// * [userID] is defined by you, so feel free to use the same ID your app uses to identify it's users.
  /// * [userType] - Were the user account will be registered.
  Future<RookUser> registerRookUser(
    String userID,
    UserType userType,
  ) async {
    await _checkAuthorization();

    final user = await _preferences.getRookUser(userType);

    if (user != null && user.id == userID) {
      _logger.info('There is a user with the same ID, returning local user...');
      return user;
    }

    final newUser = RookUser(id: userID, type: userType);

    final request = CreateRookUserRequest(
      sourceOfData: userType.sourceOfData,
      userID: userID,
    );

    await api.createRookUser(request);

    _preferences
        .saveRookUser(newUser)
        .then(
          (success) =>
              _logger.info('$userType user save in preferences: $success'),
        )
        .catchError(
          (error) => _logger.severe('Error saving room user: $error'),
        );

    return newUser;
  }

  /// Deletes the corresponding user to the provided [UserType].
  ///
  /// This does not deletes the user from server.
  Future<bool> removeUserFromPreferences(UserType userType) {
    return _preferences.deleteRookUser(userType);
  }
}
